<?php

/**
 * The main class of the plugin.
 * All the magic happens here.
 */
class AudioVideoPlugin {

	/**
	 * Callback function called for [audioplayer] tags which returns the HTML code for an audio player.
	 *
	 * @param unknown $action
	 * @param array $attr
	 *        	the attributes given in the tag
	 * @param unknown $content
	 * @param unknown $params
	 * @param unknown $node_object
	 * @return boolean|string the HTML code for an audio player
	 */
	static function getAudioHtml($action, $attr, $content, $params, $node_object) {
		if ($action == 'validate') {
			// not used for now
			return true;
		}
		return self::getPlayerHtml('audio', $attr);
	}

	// getAudioHtml()

	/**
	 * Callback function called for [videoplayer] tags which returns the HTML code for a video player.
	 *
	 * @param unknown $action
	 * @param array $attr
	 *        	the attributes given in the tag
	 * @param unknown $content
	 * @param unknown $params
	 * @param unknown $node_object
	 * @return boolean|string the HTML code for a video player
	 */
	static function getVideoHtml($action, $attr, $content, $params, $node_object) {
		if ($action == 'validate') {
			// not used for now
			return true;
		}
		return self::getPlayerHtml('video', $attr);
	}

	// getVideoHtml()

	/**
	 * Generates the HTML code for a media player element.
	 *
	 * @param string $mediatypeToUse
	 *        	the media type to use ("audio" or "video")
	 * @param array $attr
	 *        	the attributes given in the tag
	 * @return string the HTML code for a media player element (or an error message if something goes wrong)
	 */
	private static function getPlayerHtml($mediatypeToUse, $attr) {
		// check given file path
		$file = $attr ['default'];
		try {
			$filePathRel = self::getFilePath($file);
		} catch (\Exception $e) {
			return $e->getMessage();
		}
		$filePathAbs = getcwd() . '/' . $filePathRel;
		$fileUrl = BLOG_BASEURL . $filePathRel;

		// Show control elements? Set by default
		$controls = !isset($attr ['controls']) || self::isTrue($attr ['controls']) ? ' controls' : '';
		// Autoplay on load? Not set by default
		$autoplay = isset($attr ['autoplay']) && self::isTrue($attr ['autoplay']) ? ' autoplay' : '';
		// Loop play? Not set by default
		$loop = isset($attr ['loop']) && self::isTrue($attr ['loop']) ? ' loop' : '';
		// now the options for video elements only
		if ($mediatypeToUse == 'video') {
			// Which image file to set as poster? Not set by default
			try {
				$poster = isset($attr ['poster']) ? ' poster="' . self::getFilePath($attr ['poster']) . '"' : '';
			} catch (\Exception $e) {
				return $e->getMessage();
			}
			// Custom width and height
			$width = '';
			$height = '';
			$width = isset($attr ['width']) ? ' width="' . intval($attr ['width']) . '"' : '';
			$height = isset($attr ['height']) ? ' height="' . intval($attr ['height']) . '"' : '';
		}
		$html = '<' . $mediatypeToUse . ' ' . //
		'class="audiovideo" ' . //
		'type="' . mime_content_type($filePathAbs) . '" ' . //
		'src="' . $fileUrl . '"' . //
		$controls . //
		$autoplay . //
		$loop . //
		$poster . //
		$width . //
		$height . //
		'>' . //
		'Unable to play media file. <a href="' . $fileUrl . '">Download instead</a>' . //
		'</' . $mediatypeToUse . '>';
		return $html;
	}

	// getPlayerHtml()

	/**
	 * Checks if the given value means "true".
	 *
	 * @param unknown $val
	 *        	the value to be checked
	 * @return boolean <code>true</code>, if the given value means "true"; otherwise <code>false</code>
	 */
	private static function isTrue($val) {
		if (is_bool($val)) {
			return $val;
		}
		if (is_string($val)) {
			switch ($val) {
				case "true":
				case "1":
				case "yes":
				case "ja":
				case "si":
					return true;
					break;
				default:
					return false;
					break;
			}
		}
	}

	// isTrue()

	/**
	 * Returns the relative path of the given audio/video file.
	 *
	 * @param string $path
	 *        	the path to validate
	 * @throws Exception if the path is invalid
	 * @return string the relative path
	 */
	private static function getFilePath($file) {
		// prevent path traversal
		if (strpos($path, '..') !== false) {
			throw new Exception('Invalid file given');
		}
		// check if file exists
		$filePathRel = 'fp-content/' . $file;
		if (!file_exists($filePathRel)) {
			throw new Exception('Given file does not exist.');
		}
		return $filePathRel;
	}

	// getFilePath()

	/**
	 * Initializes the BBCode tags of the plugin.
	 */
	public static function initializePluginTags() {
		// get the global bbcode object
		$bbcode = plugin_bbcode_init();

		// add tag "audioplayer"
		$bbcode->addCode('audioplayer', // tag name: this will go between square brackets
		'callback_replace_single', // type of action: we'll use a callback function
		'AudioVideoPlugin::getAudioHtml', // name of the callback function
		array(
			'usecontent_param' => array(
				'default'
			)
		), // supported parameters: "default" is [acronym=valore]
		'inline', // type of the tag, inline or block, etc
		array(
			'listitem',
			'block',
			'inline',
			'link'
		), // type of elements in which you can use this tag
		array()); // type of elements where this tag CAN'T go (in this case, none, so it can go everywhere)
		$bbcode->setCodeFlag('audioplayer', 'closetag', BBCODE_CLOSETAG_FORBIDDEN); // a closing tag is forbidden (no [/tag])

		// add tag "videoplayer"
		$bbcode->addCode('videoplayer', // tag name: this will go between square brackets
		'callback_replace_single', // type of action: we'll use a callback function
		'AudioVideoPlugin::getVideoHtml', // name of the callback function
		array(
			'usecontent_param' => array(
				'default'
			)
		), // supported parameters: "default" is [acronym=valore]
		'inline', // type of the tag, inline or block, etc
		array(
			'listitem',
			'block',
			'inline',
			'link'
		), // type of elements in which you can use this tag
		array()); // type of elements where this tag CAN'T go (in this case, none, so it can go everywhere)
		$bbcode->setCodeFlag('videoplayer', 'closetag', BBCODE_CLOSETAG_FORBIDDEN); // a closing tag is forbidden (no [/tag])
	}

	// initializePluginTags()
} // class AudioVideoPlugin